#include <mix/aes.h>
#include <mix/net.h>
#include <mix/misc.h>
#include <mix/mix.h>

void aes_test (void);
void net_test (void);
void misc_test (void);

int
main ()
{
  printf ("libmix version %s found...\n", libmix_version);
  printf ("libmix test - 1: aes 2: misc 3: net > ");
  switch (getchar ())
    {
    case '1':
      aes_test ();
      break;
    case '2':
      misc_test ();
      break;
    case '3':
      net_test ();
      break;
    }
  return 0;
}

void
net_test (void)
{
  char ip[100];
  unsigned long lh;
  int i, u, t, c;
  char tst[50];
  strcpy (tst, "the transmission could be received!");
  printf ("Performing net test...\n");
  strcpy (ip, "1.2.3.4");
  if (isip (ip))
    printf ("isip test passed\n");
  lh = resolve ("localhost");
  printf ("localhost: %lu\n", lh);
  printf ("ntoa test: %s\n", ntoa (lh));
  incip (ip);
  printf ("incip: %s\tdecip: ", ip);
  decip (ip);
  printf ("%s\n", ip);
  i = rawsock (ICMP);
  u = rawsock (UDP);
  t = rawsock (TCP);
  printf ("raw sockets... icmp %s, udp %s, tcp %s\n",
	  i >= 0 ? "ok" : "failed", u >= 0 ? "ok" : "failed", t >= 0 ? "ok" : "failed");
  printf ("calling tfntransmit() (random protocol)... \n");
  aes_key ("just_testing");
  tfntransmit (lh, lh, -1, tst);
  for (c = 1; c <= 10; c++)
    {
      char *p = tfnread (i, u, t);
      printf ("tfnread(), try %d... %s\n", c, p == NULL ? "failed" : p);
      if (p != NULL)
	break;
    }
}

void
misc_test (void)
{
  char *str1 = "subst test!passed";
  char str2[100], str3[100];
  char str4[100];
  strcpy (str4, " StRlOwer tEsT... PaSsEd");

  printf ("Performing misc test...\n");
  printf ("Small random number: %ld\tlarge random number: %ld\n",
	  getrandom (0, 10), getrandom (1024, 65535));
  printf ("Installing sighandler... ");
  sighandler (SIG_IGN);
  raise (SIGHUP);
  raise (SIGINT);
  raise (SIGTRAP);
  raise (SIGIOT);
  raise (SIGTERM);
  raise (SIGCHLD);
  strscpy (str1, str2, str3, '!');
  strlower (str4);
  printf ("ok\n%s  %s\n", str2, str3);
  printf ("%s\n", str4);
  fflush (stdout);
  log ("/dev/stdout", "log function is %s\n", "ok");
}

void
aes_test (void)
{
  const u1byte *buf1 = "AES crypto test passed.";
  const u1byte *buf2 = "passwords can be up to 32 bytes.";
  const char *buf3 = "this hash can be up to 32 bytes";
  char *tmp1, *tmp2;
  int strl = strlen ((const char *) buf1);

  printf ("Performing aes test...\n");
  alg = CAST256;
  aes_key (buf2);
  tmp1 = aes_encrypt (buf1, &strl);
  tmp2 = aes_decrypt (tmp1, strl);
  printf ("\n\tAES CAST256 algorithm:\nEncrypted: %s\nDecrypted: %s\n", tmp1, tmp2);
  free (tmp1);
  free (tmp2);
  tmp1 = aes_hash (buf3);
  printf ("Hash test: %s\n", tmp1);
  free (tmp1);
  alg = MARS;
  aes_key (buf2);
  tmp1 = aes_encrypt (buf1, &strl);
  tmp2 = aes_decrypt (tmp1, strl);
  printf ("\n\tAES MARS algorithm:\nEncrypted: %s\nDecrypted: %s\n", tmp1, tmp2);
  free (tmp1);
  free (tmp2);
  tmp1 = aes_hash (buf3);
  printf ("Hash test: %s\n", tmp1);
  free (tmp1);

  alg = SAFERP;
  aes_key (buf2);
  tmp1 = aes_encrypt (buf1, &strl);
  tmp2 = aes_decrypt (tmp1, strl);
  printf ("\n\tAES SAFERPLUS algorithm:\nEncrypted: %s\nDecrypted: %s\n", tmp1, tmp2);
  free (tmp1);
  free (tmp2);
  tmp1 = aes_hash (buf3);
  printf ("Hash test: %s\n", tmp1);
  free (tmp1);

  alg = TWOFISH;
  aes_key (buf2);
  tmp1 = aes_encrypt (buf1, &strl);
  tmp2 = aes_decrypt (tmp1, strl);
  printf ("\n\tAES TWOFISH algorithm:\nEncrypted: %s\nDecrypted: %s\n", tmp1, tmp2);
  free (tmp1);
  free (tmp2);
  tmp1 = aes_hash (buf3);
  printf ("Hash test: %s\n", tmp1);
  free (tmp1);

  alg = RIJNDAEL;
  aes_key (buf2);
  tmp1 = aes_encrypt (buf1, &strl);
  tmp2 = aes_decrypt (tmp1, strl);
  printf ("\n\tAES RIJNDAEL algorithm:\nEncrypted: %s\nDecrypted: %s\n", tmp1, tmp2);
  free (tmp1);
  free (tmp2);
  tmp1 = aes_hash (buf3);
  printf ("Hash test: %s\n", tmp1);
  free (tmp1);
}
