#include <mix/misc.h>
#include <mix/mix.h>

char rseed[65535];
int rcounter = 0;
int bufsize = 1024;

void
random_init (void)
{
  int rfd = open ("/dev/arandom", O_RDONLY);
  if (rfd < 0)
    rfd = open ("/dev/urandom", O_RDONLY);
  if (rfd < 0)
    rfd = open ("/dev/random", O_RDONLY);
  if (rfd >= 0)
    {
      rcounter = read (rfd, rseed, 65535);
      close (rfd);
    }
  else
    {
    rcounter = 65535;
    memset (rseed, (getpid () % 0xf0) + 1, 65535);
    }

#ifdef HAVE_INITSTATE
 initstate(getpid(), rseed, 65535);
#endif

}

long
getrandom (int min, int max)
{
  if (rcounter < 2)
    random_init ();
  srandom (rseed[rcounter] + (rseed[rcounter - 1] << 8));
  rcounter -= 2;
  return ((random () % (int) (((max) + 1) - (min))) + (min));
}

void 
sighandler (void *handler)
{
  signal (SIGHUP, handler);
  signal (SIGINT, handler);
  signal (SIGFPE, handler);
  signal (SIGBUS, handler);
  signal (SIGTRAP, handler);
  signal (SIGIOT, handler);
  signal (SIGPIPE, handler);
  signal (SIGTTIN, handler);
  signal (SIGTTOU, handler);
  signal (SIGTERM, handler);
  signal (SIGCHLD, handler);
}

void
strchop (char *str)
{
  int i;

  for (i = 0; i < (int) strlen (str) - 2; i++)
    if ((str[i] == '\n') || (str[i] == '\r'))
      str[i] = '\0';

}

void
strlower (char *str)
{
  int i;
  for (i = 0; i < (int) strlen (str); i++)
    if (isupper (str[i]))
      str[i] = tolower (str[i]);
}

void
strnsubst (char *str, char subst, int siz)
{
  int i;
  for (i = 0; i < (siz - 1); i++)
    if (str[i] == '\0')
      str[i] = subst;
}

char *
strscpy (char *src, char *sdest, char *edest, char c)
{
  size_t i;

  for (i = 0; i <= strlen (src); i++)
    if (src[i] == c)
      break;

  memcpy (sdest, src, i);       /* copy this anyways, 2nd arg is optional */

  if (i >= strlen (src))
    return NULL;

  if (edest != NULL)
    memcpy (edest, src + i + 1, strlen (src) - i - 1);

  return (sdest);
}

void
log (char *log, char *msg,...)
{
  FILE *fd = fopen (log, "a");
  va_list va;

  if (fd == NULL)
    return;

  va_start (va, msg);
  vfprintf (fd, msg, va);
  va_end (va);

  fclose (fd);
}

void
ioterm (int infd, int outfd, int crypt)
{
  char *buf, *outbuf;
  fd_set rfds;
  int i;

  buf = (char *) malloc (bufsize);

  while (1)
    {
      FD_ZERO (&rfds);
      FD_SET (infd, &rfds);
      FD_SET (outfd, &rfds);

      if (select ((infd > outfd ? infd : outfd) + 1, &rfds, NULL, NULL, NULL) < 1)
	goto bye;

      if (FD_ISSET (outfd, &rfds))
	{
	  memset (buf, 0, sizeof (buf));
	  if ((i = read (outfd, buf, sizeof (buf))) < 1)
	    goto bye;
	  if (crypt)		/* out: plaintext */
	    {
	      outbuf = aes_decrypt ((u1byte *) buf, i);
	      write (infd, outbuf, strlen (outbuf));
	      free (outbuf);
	    }
	  else
	    write (infd, buf, i);
	}

      if (FD_ISSET (infd, &rfds))
	{
	  memset (buf, 0, sizeof (buf));
	  if ((i = read (infd, buf, sizeof (buf))) < 1)
	    goto bye;
	  if (crypt)		/* in: encrypted */
	    {
	      outbuf = aes_encrypt ((u1byte *) buf, &i);
	      write (outfd, outbuf, aes_binary ? i : strlen (outbuf));
	      free (outbuf);
	    }
	  else
	    write (outfd, buf, i);
	}
    }
bye:
  free (buf);
  close (infd);
  close (outfd);
}

int
pattern (char *data, char **list)
{
  int i = 0;

  while (strcmp (list[i++], NULL) != 0)
    if (strstr (data, list[i]) != NULL)
        return 1;

  return 0;
}

