#ifndef MIX_H
#define MIX_H

#include <mix/aes.h>
#include <mix/misc.h>
#include <mix/net.h>

#define libmix_version "v1.05"

/* advanced encryption standard interface */

#define CAST256         0x01    /* medium speed, big keyspace */
#define MARS            0x02    /* fast speed, medium keyspace */
#define SAFERP          0x03    /* slow speed, extremely big keyspace */
#define TWOFISH         0x04    /* fast speed, medium keyspace */
#define RIJNDAEL        0x05    /* medium-fast speed, medium-big keyspace */

extern short alg;               /* algorithm selector */
extern short aes_binary;        /* to disable base64 encoding */

u4byte *aes_key (const char *);			/* set a key */
u1byte *aes_encrypt (const u1byte *in, int *);  /* return encrypted data */
u1byte *aes_decrypt (const u1byte *, int);      /* return decrypted data */
u1byte *aes_hash (const s1byte *);              /* return AES digest */
s1byte *base64_in (const u1byte *, int);        /* return base64'd data */
u1byte *base64_out (const s1byte *, int);       /* return un-base64'd data */

/* miscellaneous functions interface */

long getrandom (int, int); /* return random number between arg1 and arg2 */
void sighandler (void *); /* install sighandler for common interrupt events */
void strchop (char *); /* cut string at first newline with \0 */
void strlower (char *); /* convert all uppercase alpha to lowercase alpha */
void strnsubst (char *, char, int); /* substitute \0's in a string */
char *strscpy (char *, char*, char *, char); /* separate a string */
void ioterm (int, int, int);   /* 3rd variable selects encryption */
int pattern (char *, char **); /* list should always have 2 trailing NULLs */
void log (char *, char *,...); /* try logging arg2-N into file arg1 */

/* network functions interface */

int isip (char *);
unsigned long resolve (char *);
char *ntoa (unsigned int);
unsigned short sum (unsigned short *, int);
int incip (char *);
int decip (char *);
int rawsock (int);
void tfntransmit (unsigned long, unsigned long, int, char *);
char * tfnread (int, int, int); /* i, u, t */

#endif
